(function ($) {
  /**
   * Retrieves content from endpoints in order to display Bean blocks.
   */
  Drupal.behaviors.OpenEDUSyndicationBean = {
    // Set global variables.
    variables: {
      timeout: 900000, // 15min
      txtTime: 'OpenEDUSyndication_time_',
      txtData: 'OpenEDUSyndication_data_'
    },

    // Check if a string is empty.
    isEmpty: function(str) {
      return (!str || 0 === str.length);
    },

    // Create hash code based on a string.
    hashCode: function(s){
      return s.split("").reduce(function(a,b){a=((a<<5)-a)+b.charCodeAt(0);return a&a},0);
    },

    // Remove storage.
    storageClear: function(namespace) {
      // Remove data.
      localStorage.removeItem(this.variables.txtData + namespace);

      // Remove time.
      localStorage.removeItem(this.variables.txtTime + namespace);
    },

    // Check if storage is still valid.
    storageCheck: function(namespace) {
      var itemTime = localStorage.getItem(this.variables.txtTime + namespace),
          itemData = localStorage.getItem(this.variables.txtData + namespace);

      // See if our previous request timed out.
      if (this.isEmpty(itemData) || this.isEmpty(itemTime) || ((Date.now() - itemTime) > this.variables.timeout)) {
        // Clear namespace storage.
        this.storageClear(namespace);

        // Return false so data will be renewed.
        return false;
      }

      return itemData;
    },

    // Set storage.
    storageSet: function(namespace, string) {
      // Set data.
      localStorage.setItem(this.variables.txtData + namespace, string);

      // Set time.
      localStorage.setItem(this.variables.txtTime + namespace, Date.now());
    },

    // Drupal attach behavior.
    attach: function (context, settings) {
      // Set vars.
      var that = this,
          OpenEDUBean = {
            endpoint: settings.basePath + settings.OpenEDUSyndicationBean.endpoint,
            items: $('.openedu-syndication-client-render')
          };

      // Check if we have items available.
      if (OpenEDUBean.items.length) {
        $.each(OpenEDUBean.items, function() {
          var $this = $(this),
              attrs = $this.data(),
              nameSpace = that.hashCode(OpenEDUBean.endpoint + JSON.stringify(attrs)),
              cahedResult = that.storageCheck(nameSpace);

          if (cahedResult) {
            $this.html(cahedResult);
          }
          else {
            // URL is mandatory.
            if (attrs.url != '') {
              // Submit request.
              $.ajax({
                url: attrs.request_url,
                // data: attrs
              })
              .done(function(response) {
                response = $(response).find('result').text();
                var txtNotFound = Drupal.t('No results found.'),
                    txtResponse = $.trim(response.replace(/(<\S([^>]+)>)/ig,''));

                // Only set storage if results differ from txtNotFound.
                if (txtNotFound != txtResponse) {
                  that.storageSet(nameSpace, response);
                }

                // Set html.
                $this.html(response);
              })
              .fail(function() {
                console.log('Failed while loading:');
                console.log($this);
              })
              .always(function() {
                //
              });
            }
          }
        });
      }

    }
  };
})(jQuery);
