<?php

/**
 * Register Syndication Server form.
 */
function openedu_syndication_client_register_server_form($form, &$form_state) {
  // Set vars.
  $context_id = openedu_syndication_og_context_id_load();
  $openedu_syndication_client_var = variable_get(OPENEDU_SYNDICATION_CLIENT_VAR_GENERAL, array());

  // Check if variable that relates to the actual context is available.
  $openedu_syndication_client_local = isset($openedu_syndication_client_var['endpoint' . $context_id]) ?
    $openedu_syndication_client_var['endpoint' . $context_id] : FALSE;

  // Register server.
  $form['server'] = array(
    '#type' => 'fieldset',
    '#title' => t('Register Syndication Server'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['server']['url'] = array(
    '#type' => 'textfield',
    '#title' => t('URL'),
    '#description' => t('Insert the endpoint URL that will be used to retrieve all syndicated data across installations. Ex. <i>http://endpoint.url</i>'),
    '#field_suffix' => OPENEDU_SYNDICATION_ENDPOINT_SERVER,
    '#default_value' => (isset($openedu_syndication_client_local['url']) ?
      $openedu_syndication_client_local['url'] : FALSE),
    '#required' => TRUE,
  );
  $form['server']['private_token'] = array(
    '#type' => 'textfield',
    '#title' => t('Private Token'),
    '#description' => t('Request will be validated against the token generated by the server.'),
    '#default_value' => (isset($openedu_syndication_client_local['private_token']) ?
      $openedu_syndication_client_local['private_token'] : FALSE),
    '#required' => TRUE,
  );
  $form['server']['public_token'] = array(
    '#type' => 'textfield',
    '#title' => t('Public Token'),
    '#description' => t('Request will be validated against the token generated by the server.'),
    '#default_value' => (isset($openedu_syndication_client_local['public_token']) ?
      $openedu_syndication_client_local['public_token'] : FALSE),
    '#required' => TRUE,
  );

  // Enforce validate and submit.
  $form['#validate'][] = 'openedu_syndication_client_register_server_form_validate';
  $form['#submit'][] = 'openedu_syndication_client_register_server_form_submit';

  return system_settings_form($form);
}

/**
 * Register Syndication Server form validate.
 * In order to validate endpoint a request will be send over waiting for a
 * HTTP 200 status code.
 */
function openedu_syndication_client_register_server_form_validate(&$form, &$form_state) {
  // Set vars.
  $url = $form_state['values']['url'];
  $token = $form_state['values']['private_token'];

  // Check if endpoint should be validated or not.
  if (TRUE == openedu_syndication_endpoint_validation()) {
    // Submit request.
    $request = openedu_authentication_http_request($url . OPENEDU_SYNDICATION_ENDPOINT_SERVER, array(), 'retrieve', $token);

    // Check for 200 and save response from server.
    if (200 === (int) $request->code) {
      variable_set(OPENEDU_SYNDICATION_CLIENT_VAR_ENDPOINTS, json_decode($request->data));
    }
    // Set feedback saying something went wrong.
    else {
      form_set_error('url', t('Sorry, we couldn\'t validate the endpoint properly please, try again or review if the information provided is valid.'));
    }
  }
}

/**
 * Register Syndication Server form submit.
 */
function openedu_syndication_client_register_server_form_submit(&$form, &$form_state) {
  // Load variable.
  $context_id = openedu_syndication_og_context_id_load();
  $openedu_syndication_client_var = variable_get(OPENEDU_SYNDICATION_CLIENT_VAR_GENERAL, array());

  // Update variable.
  variable_set(OPENEDU_SYNDICATION_CLIENT_VAR_GENERAL, array(
    'endpoint' . $context_id => array(
      'url' => $form_state['values']['url'],
      'private_token' => $form_state['values']['private_token'],
      'public_token' => $form_state['values']['public_token']
    )
  ) + $openedu_syndication_client_var);
}

/**
 * Register content types form.
 * Register all content types that should be shared through syndication.
 */
function openedu_syndication_client_content_types_form($form, &$form_state) {
  // Ser vars.
  $context_id = openedu_syndication_og_context_id_load();
  $openedu_syndication_client_var = variable_get(OPENEDU_SYNDICATION_CLIENT_VAR_GENERAL, array());

  // Check if variable that relates to the actual context is available.
  $openedu_syndication_client_local = isset($openedu_syndication_client_var['content_types' . $context_id]) ?
    $openedu_syndication_client_var['content_types' . $context_id] : FALSE;

  // List content types.
  $form['content'] = array(
    '#type' => 'fieldset',
    '#title' => t('Content types enabled for sharing.'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  // Display all options so users can set which ones should be shared.
  $form['content']['types'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Select all content types that should be accessible through Syndication.'),
    '#options' => variable_get(OPENEDU_SYNDICATION_VAR_CONTENT_TYPES, array()),
    '#default_value' => (is_array($openedu_syndication_client_local) ?
      array_filter($openedu_syndication_client_local) : array())
  );

  // Enforce validate and submit.
  $form['#submit'][] = 'openedu_syndication_client_content_types_form_submit';

  return system_settings_form($form);
}

/**
 * Register content types form submit.
 *
 * Once a content type is enabled we need to ensure the field instance is added
 * to it and case being disabled, ensure field instance is being removed.
 */
function openedu_syndication_client_content_types_form_submit($form, &$form_state) {
  // Cleaning array.
  $context_id = openedu_syndication_og_context_id_load();
  $safe_values = array_filter($form_state['values']['types']);

  // Load global client variable.
  $openedu_syndication_client_var = variable_get(OPENEDU_SYNDICATION_CLIENT_VAR_GENERAL, array());

  // Load local server endpoint info, related to actual context.
  $context_endpoint_info = isset($openedu_syndication_client_var['endpoint' . $context_id]) ?
    $openedu_syndication_client_var['endpoint' . $context_id] : FALSE;

  // Update variable.
  variable_set(OPENEDU_SYNDICATION_CLIENT_VAR_GENERAL, array(
    'content_types'. $context_id => $safe_values,
  ) + $openedu_syndication_client_var);

  // Submit new data to the server.
  if (!empty($context_endpoint_info)) {
    // Set variables.
    $server_url = $context_endpoint_info['url'] . OPENEDU_SYNDICATION_ENDPOINT_SERVER;
    $server_token = $context_endpoint_info['private_token'];

    // Submit request.
    openedu_authentication_http_request($server_url . '/' . $server_token, array('data' => $safe_values), 'update', $server_token);

    // @TODO: Validate answer and create fallback.
  }

  // Set feedback.
  drupal_set_message(t('The configuration options have been saved.'));

  // Redirect user.
  drupal_goto(OPENEDU_SYNDICATION_ADMIN_PATH . '/client/content-types');
}
