<?php

/**
 * @file
 * Views hooks implemented for the OpenEDU Syndication Client module.
 */

/**
 * Implements hook_views_data_alter().
 */
function openedu_syndication_client_views_data_alter(&$data) {
  if (isset($data['node']) && !isset($data['node']['syndicated_types'])) {
    $data['node']['syndicated_types'] = array(
      'title' => t('Syndicated content types'),
      'help' => t('Content types enabled via OpenEDU Syndication.'),
      'filter' => array(
        'handler' => 'openedu_syndication_client_handler_filter_syndicated_types',
      ),
    );
  }
}

/**
 * Implements hook_views_data().
 */
function openedu_syndication_client_views_data() {
  $table = array(
    'openedu_syndication_client_syndicated_items' => array(
      'table' => array(
        'group' => 'OpenEDU Syndication',
        'base' => array(
          'field' => 'sid',
          'title' => 'OpenEDU Syndication',
          'help' => 'OpenEDU Syndication Client: Syndicated Items.'
        )
      ),
      'sid' => array(
        'title' => t('Syndicated item id.'),
        'help' => '',
        'field' => array(
          'click sortable' => TRUE,
        ),
        'filter' => array(
          'handler' => 'views_handler_filter_numeric'
        ),
        'sort' => array(
          'handler' => 'views_handler_sort'
        )
      ),
      'user_id' => array(
        'title' => t('User ID who syndicated item.'),
        'help' => '',
        'relationship' => array(
          'base' => 'users',
          'base field' => 'uid',
          'handler' => 'views_handler_relationship',
          'label' => t('OpenEDU Syndication Client: USER LABEL'),
          'title' => t('OpenEDU Syndication Client: USER TITLE'),
          'help' => t('User ID who Syndicated content.'),
        ),
      ),
      'entity_id' => array(
        'title' => t('Entity ID being syndicated.'),
        'help' => '',
        // @TODO: Right now entity_id will only relate with nids, find best
        // solution to allow field to related to any entity/bundle.
        'relationship' => array(
          'base' => 'node',
          'base field' => 'nid',
          'handler' => 'views_handler_relationship',
          'label' => t('OpenEDU Syndication Client: NID LABEL'),
          'title' => t('OpenEDU Syndication Client: NID TITLE'),
          'help' => t('OpenEDU Syndication Client: NID HELP'),
        ),
      ),
      'entity' => array(
        'title' => t('Entity type being syndicated.'),
        'help' => '',
        'field' => array(
          'click sortable' => TRUE,
        ),
        'filter' => array(
          'handler' => 'views_handler_filter_string'
        ),
        'sort' => array(
          'handler' => 'views_handler_sort'
        )
      ),
      'bundle' => array(
        'title' => t('Bundle type being syndicated.'),
        'help' => '',
        'field' => array(
          'click sortable' => TRUE,
        ),
        'filter' => array(
          'handler' => 'views_handler_filter_string'
        ),
        'sort' => array(
          'handler' => 'views_handler_sort'
        )
      ),
      'group_id' => array(
        'title' => t('OG Group ID syndicated belongs to.'),
        'help' => '',
        'field' => array(
          'click sortable' => TRUE,
        ),
        'filter' => array(
          'handler' => 'views_handler_filter_numeric'
        ),
        'sort' => array(
          'handler' => 'views_handler_sort'
        )
      ),
      'created' => array(
        'title' => t('When context was assigned to be syndicated.'),
        'help' => '',
        'field' => array(
          'handler' => 'views_handler_field_date',
          'click sortable' => TRUE,
        ),
        'sort' => array(
          'handler' => 'views_handler_sort_date',
        ),
        'filter' => array(
          'handler' => 'views_handler_filter_date',
        ),
      ),
    )
  );

  return $table;
}
